@@ -3,6 +3,8 @@ from __future__ import annotations
 from functools import cached_property
 from typing import TYPE_CHECKING
 
+from rich.markup import escape
+
 
 if TYPE_CHECKING:
     from pathlib import Path
@@ -17,7 +19,7 @@ class BaseNode:
     """
 
     def __init__(self, name: str, path: Path):
-        self.name = name
+        self.name = escape(name)
         self.path = path
 
         self.specs: list[NodeSpec] = []  # matched in ``match_specs``
@@ -53,15 +55,20 @@ class BaseNode:
 
         self.specs = [spec for spec in specs if spec.match(self)]
 
-    def spec_attr(self, attr: str) -> Any:
+    def spec_attr(self, attr: str, coalesce: bool = False) -> Any:
         """
         Get the requested attribute from the first matching spec to provide it.
 
         :param attr: the requested attribute
+        :param coalesce: whether to group attrs from all specs and return a list
         :return: the value of the attribute if found, ``None`` otherwise
         """
 
+        values = []
         for spec in self.specs:
             if attr_val := getattr(spec, attr, None):
-                return attr_val
-        return None
+                if coalesce:
+                    values.append(attr_val)
+                else:
+                    return attr_val
+        return values if coalesce else None
